<?php

namespace WpBeacon\Providers;

use Throwable;
use WpBeacon\Actions\Api\GetCoreVersionsAction;
use WpBeacon\Actions\Api\GetPluginVersionsAction;
use WpBeacon\Actions\Api\GetThemeVersionsAction;

class ApiServiceProvider implements Provider
{
    const ROUTE_NAMESPACE = 'beacon/v1';
    const ROUTE = '/versions';

    public function register(): void
    {
        add_action('rest_api_init', function () {
            register_rest_route(self::ROUTE_NAMESPACE, self::ROUTE, [
                'methods' => 'GET',
                'callback' => function () {
                    return self::getEndpointData();
                },
                'permission_callback' => function () {
                    return current_user_can(MonitoringUserProvider::CAPABILITY);
                },
            ]);
        });

        add_filter('rest_authentication_errors', [$this, 'conditionallyEnableApplicationPasswords']);
    }

    private static function getEndpointData(): array
    {
        $data = [
            'wordpress' => [],
            'plugins' => [],
            'themes' => [],
            'php' => PHP_VERSION,
            'db' => null,
        ];

        try {
            $data['wordpress'] = GetCoreVersionsAction::getCoreVersions();
            $data['plugins'] = GetPluginVersionsAction::all();
            $data['themes'] = GetThemeVersionsAction::all();
            $data['db'] = self::getDatabaseVersion();
        } catch (Throwable $e) {
            error_log($e->getMessage());
        }

        return $data;
    }

    private static function getDatabaseVersion(): string
    {
        global $wpdb;

        return $wpdb->db_server_info();
    }

    public function conditionallyEnableApplicationPasswords($result)
    {
        $request_uri = add_query_arg([]);

        $current_route = '/' . rest_get_url_prefix() . '/' . self::ROUTE_NAMESPACE . self::ROUTE;

        if ($request_uri !== $current_route) {
            return $result;
        }

        add_filter('wp_is_application_passwords_available', '__return_true', 200);

        return $result;
    }
}
