<?php

namespace WpBeacon\Providers;

use WpBeacon\Support\PluginVersion;
use WpBeacon\Migrations\Migrations;

class UpdateServiceProvider implements Provider
{
    public function register(): void
    {
        $currentVersion = PluginVersion::get();

        if (version_compare($currentVersion, WpBeaconServiceProvider::PLUGIN_VERSION, 'eq')) {
            return;
        }

        $this->migrate($currentVersion);
    }

    private function migrate($currentVersion)
    {
        $migrations = [];

        foreach (Migrations::all() as $migration) {
            if ($migration->applies($currentVersion)) {
                $migrations[] = $migration;
            }
        }

        if (empty($migrations)) {
            return;
        }

        global $wpdb;
        $wpdb->query('START TRANSACTION');

        foreach ($migrations as $migration) {
            $migration->run();
        }
        $wpdb->query('COMMIT');

        PluginVersion::update();
    }
}
